﻿using Hangfire.Dashboard;
using Microsoft.Extensions.DependencyInjection;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using Volo.Abp.Users;

namespace Sanhe.Abp.Hangfire.Dashboard.Authorization
{
    /// <summary>
    /// Hangfire仪表盘权限过滤器
    /// </summary>
    public class DashboardAuthorizationFilter : IDashboardAsyncAuthorizationFilter
    {
        private readonly string[] _requiredPermissionNames;

        public DashboardAuthorizationFilter(params string[] requiredPermissionNames)
        {
            _requiredPermissionNames = requiredPermissionNames;
        }

        public async Task<bool> AuthorizeAsync(DashboardContext context)
        {
            // 如果未登录则没有权限访问
            if (!IsLoggedIn(context))
            {
                return false;
            }

            if (_requiredPermissionNames.IsNullOrEmpty())
            {
                return true;
            }

            return await IsPermissionGrantedAsync(context, _requiredPermissionNames);
        }

        private static bool IsLoggedIn(DashboardContext context)
        {
            var currentUser = context.GetHttpContext().RequestServices.GetRequiredService<ICurrentUser>();

            return currentUser.IsAuthenticated;
        }

        private static async Task<bool> IsPermissionGrantedAsync(DashboardContext context, string[] requiredPermissionNames)
        {
            var permissionChecker = context.GetHttpContext().RequestServices.GetRequiredService<IDashboardPermissionChecker>();

            return await permissionChecker.IsGrantedAsync(context, requiredPermissionNames);
        }
    }
}
