﻿using Hangfire;
using Hangfire.PostgreSql;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Volo.Abp.Hangfire;
using Volo.Abp.Modularity;

namespace Sanhe.Abp.Hangfire.Storage.PostgreSql;

[DependsOn(typeof(AbpHangfireModule))]
public class AbpHangfireStoragePostgreSqlModule : AbpModule
{
    public override void PreConfigureServices(ServiceConfigurationContext context)
    {
        var configuration = context.Services.GetConfiguration();

        var postgreSqlStorageOptions = new PostgreSqlStorageOptions();
        configuration.GetSection("Hangfire:PostgreSql").Bind(postgreSqlStorageOptions);

        var hangfirePostgreSqlConfiguration = configuration.GetSection("Hangfire:PostgreSql:Connection");
        var hangfirePostgreSqlConnection = hangfirePostgreSqlConfiguration.Exists()
            ? hangfirePostgreSqlConfiguration.Value
            : configuration.GetConnectionString("Default");

        var jobStorage = new PostgreSqlStorage(hangfirePostgreSqlConnection, postgreSqlStorageOptions);
        context.Services.AddSingleton<JobStorage, PostgreSqlStorage>(factory => jobStorage);

        PreConfigure<IGlobalConfiguration>(config => config.UseStorage(jobStorage));
    }
}
