﻿using Microsoft.AspNetCore.Mvc;
using System;
using System.Threading.Tasks;
using Volo.Abp;
using Volo.Abp.Application.Dtos;
using Volo.Abp.AspNetCore.Mvc;

namespace Sanhe.Abp.LocalizationManagement;

[RemoteService(Name = LocalizationRemoteServiceConsts.RemoteServiceName)]
[Area("localization")]
[Route("api/localization/resources")]
public class ResourceController : AbpController, IResourceAppService
{
    private readonly IResourceAppService _service;

    public ResourceController(IResourceAppService service)
    {
        _service = service;
    }

    [HttpPost]
    public virtual Task<ResourceDto> CreateAsync(CreateOrUpdateResourceInput input)
    {
        return _service.CreateAsync(input);
    }

    [HttpDelete]
    [Route("{id}")]
    public virtual Task DeleteAsync(Guid id)
    {
        return _service.DeleteAsync(id);
    }

    [HttpGet]
    [Route("all")]
    public virtual Task<ListResultDto<ResourceDto>> GetAllAsync()
    {
        return _service.GetAllAsync();
    }

    [HttpGet]
    [Route("{id}")]
    public virtual Task<ResourceDto> GetAsync(Guid id)
    {
        return _service.GetAsync(id);
    }

    [HttpGet]
    public virtual Task<PagedResultDto<ResourceDto>> GetListAsync(GetResourcesInput input)
    {
        return _service.GetListAsync(input);
    }

    [HttpPut]
    [Route("{id}")]
    public virtual Task<ResourceDto> UpdateAsync(Guid id, CreateOrUpdateResourceInput input)
    {
        return _service.UpdateAsync(id, input);
    }
}
