# Nuget包文件夹
$packFolder = (Get-Item -Path "./" -Verbose).FullName
# 根文件夹
$rootFolder = Join-Path $packFolder "../"

function Write-Info   
{
	param(
        [Parameter(Mandatory = $true)]
        [string]
        $text
    )

	Write-Host $text -ForegroundColor Black -BackgroundColor Green

	try 
	{
	   $host.UI.RawUI.WindowTitle = $text
	}		
	catch 
	{
		#Changing window title is not suppoerted!
	}
}

function Write-Error   
{
	param(
        [Parameter(Mandatory = $true)]
        [string]
        $text
    )

	Write-Host $text -ForegroundColor Red -BackgroundColor Black 
}

function Seperator   
{
	Write-Host ("_" * 100)  -ForegroundColor gray 
}

function Get-Current-Version { 
	$commonPropsFilePath = resolve-path "../common.props"
	$commonPropsXmlCurrent = [xml](Get-Content $commonPropsFilePath ) 
	$currentVersion = $commonPropsXmlCurrent.Project.PropertyGroup.Version.Trim()
	return $currentVersion
}

function Get-Current-Branch {
	return git branch --show-current
}	   

function Read-File {
	param(
        [Parameter(Mandatory = $true)]
        [string]
        $filePath
    )
		
	$pathExists = Test-Path -Path $filePath -PathType Leaf
	if ($pathExists)
	{
		return Get-Content $filePath		
	}
	else{
		Write-Error  "$filePath path does not exist!"
	}
}

# 解决方案列表
$solutions = (
    # Sanhe.Abp.Framework
    "./"
)

# 项目列表
$projects = (
    # modules/common
    "modules/common/Sanhe.Abp.EntityFrameworkCore",
    "modules/common/Sanhe.Abp.ExceptionHandling",
    "modules/common/Sanhe.Abp.ExceptionHandling.Emailing",
    "modules/common/Sanhe.Abp.Features.LimitValidation",
    "modules/common/Sanhe.Abp.Features.LimitValidation.Redis",
    "modules/common/Sanhe.Abp.Features.LimitValidation.Redis.Client",
    "modules/common/Sanhe.Abp.Hangfire.Dashboard",
    "modules/common/Sanhe.Abp.Hangfire.Storage.PostgreSql",
    "modules/common/Sanhe.Abp.IdGenerator",
    "modules/common/Sanhe.Abp.Localization.Dynamic",
    "modules/common/Sanhe.Abp.Notifications",
    "modules/common/Sanhe.Abp.PinyinConverter",
    "modules/common/Sanhe.Abp.PinyinConverter.ToolGoodWords",
    "modules/common/Sanhe.Abp.RealTime",
    "modules/common/Sanhe.Abp.Wrapper",
        
    # modules/elasticsearch
    "modules/elasticsearch/Sanhe.Abp.Elasticsearch",

    # modules/auditing
    "modules/auditing/Sanhe.Abp.Auditing.Application",
    "modules/auditing/Sanhe.Abp.Auditing.Application.Contracts",
    "modules/auditing/Sanhe.Abp.Auditing.HttpApi",
    "modules/auditing/Sanhe.Abp.AuditLogging",
    "modules/auditing/Sanhe.Abp.AuditLogging.Elasticsearch",
    "modules/auditing/Sanhe.Abp.AuditLogging.EntityFrameworkCore",

    # modules/logging
    "modules/logging/Sanhe.Abp.Logging",
    "modules/logging/Sanhe.Abp.Logging.Serilog.Elasticsearch",
    "modules/logging/Sanhe.Abp.Serilog.Enrichers.Application",
    "modules/logging/Sanhe.Abp.Serilog.Enrichers.UniqueId",

    # modules/identity
    "modules/identity/Sanhe.Abp.Identity.Application",
    "modules/identity/Sanhe.Abp.Identity.Application.Contracts",
    "modules/identity/Sanhe.Abp.Identity.Domain",
    "modules/identity/Sanhe.Abp.Identity.Domain.Shared",
    "modules/identity/Sanhe.Abp.Identity.EntityFrameworkCore",
    "modules/identity/Sanhe.Abp.Identity.HttpApi",
    "modules/identity/Sanhe.Abp.Identity.HttpApi.Client",

    # modules/localization-management
    "modules/localization-management/Sanhe.Abp.LocalizationManagement.Application",
    "modules/localization-management/Sanhe.Abp.LocalizationManagement.Application.Contracts",
    "modules/localization-management/Sanhe.Abp.LocalizationManagement.Domain",
    "modules/localization-management/Sanhe.Abp.LocalizationManagement.Domain.Shared",
    "modules/localization-management/Sanhe.Abp.LocalizationManagement.EntityFrameworkCore",
    "modules/localization-management/Sanhe.Abp.LocalizationManagement.HttpApi",

    # modules/menu-management
    "modules/menu-management/Sanhe.Abp.MenuManagement.Application",
    "modules/menu-management/Sanhe.Abp.MenuManagement.Application.Contracts",
    "modules/menu-management/Sanhe.Abp.MenuManagement.Domain",
    "modules/menu-management/Sanhe.Abp.MenuManagement.Domain.Identity",
    "modules/menu-management/Sanhe.Abp.MenuManagement.Domain.Shared",
    "modules/menu-management/Sanhe.Abp.MenuManagement.EntityFrameworkCore",
    "modules/menu-management/Sanhe.Abp.MenuManagement.HttpApi",

    # modules/mvc
    "modules/mvc/Sanhe.Abp.AspNetCore.Mvc.Wrapper"
)
