﻿using BookStore.Services;
using BookStore.Services.Dtos;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Threading.Tasks;
using Volo.Abp.Application.Dtos;
using Volo.Abp.AspNetCore.Mvc;

namespace BookStore.Controllers;

[Route("api/books")]
public class BookController: AbpController, IBookAppService
{
    private readonly IBookAppService _bookAppService;

    public BookController(IBookAppService bookAppService)
    {
        _bookAppService = bookAppService;
    }

    [HttpPost]
    public Task<BookDto> CreateAsync(BookCreateUpdateInput input)
    {
        return _bookAppService.CreateAsync(input);
    }

    [HttpDelete("{id}")]
    public Task DeleteAsync(Guid id)
    {
        return _bookAppService.DeleteAsync(id);
    }

    [HttpGet("{id}")]
    public Task<BookDto> GetAsync(Guid id)
    {
        return _bookAppService.GetAsync(id);
    }

    [HttpGet]
    public Task<PagedResultDto<BookDto>> GetListAsync(PagedAndSortedResultRequestDto input)
    {
        return _bookAppService.GetListAsync(input);
    }

    [HttpPut("{id}")]
    public Task<BookDto> UpdateAsync(Guid id, BookCreateUpdateInput input)
    {
        return _bookAppService.UpdateAsync(id, input);
    }
}
